package com.fluig;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Map;
import java.util.Properties;

import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.totvs.technology.foundation.alert.sender.AlertSenderApp;
import com.totvs.technology.foundation.alert.service.AlertService;
import com.totvs.technology.foundation.alert.vo.AlertVO;
import com.totvs.technology.foundation.common.exception.FDNException;

/**
 * Esta é a classe que será chamada pela Central de Notificações para enviar
 * alertas por este aplicativo.
 * 
 * Esta classe deve, obrigatoriamente, implemantar a interface "com.totvs.technology.foundation.alert.sender.AlertSenderApp",
 * que faz parte da API da Central de Notificações do Fluig.
 * 
 * @author caio.psousa@fluig.com
 *
 */
@Stateless(name = TxtSenderService.JNDI_NAME, mappedName = TxtSenderService.JNDI_NAME)
public class TxtSenderServiceBean implements TxtSenderService, AlertSenderApp {
	
	private static final Logger LOG = LoggerFactory.getLogger(TxtSenderServiceBean.class);

	/**
	 * Este serviço faz parte da API de Notificações do Fluig.
	 * Ela pode ser utilizada para consultar os dados de um alerta.
	 */
	@EJB(lookup = AlertService.JNDI_REMOTE_NAME)
	private AlertService alertService;

	/**
	 * Este é o método que será invocado pela Central de Notificações para enviar
	 * notificações através deste aplicativo.
	 * 
	 */
	@Override
	@TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
	public void sendAlert(final Long alertId) throws FDNException {
		
		try {
			
			/*
			 * Recupera os dados do alerta
			 */
			final AlertVO alert = alertService.findFullVOById(alertId);
			/*
			 * Recupera os metadados do alerta (caso não existam, retorna um map vazio)
			 */
			final Map<String, String> metadata = alertService.getMetadata(alert.getId());
			
			/*
			 * ATENÇÃO: O código a seguir é a implementaçao do envio do alerta pelo aplicativo.
			 * No caso deste exemplo específico, o alerta será salvo em um arquivo texto,
			 * porém a implementação depende do propósito de cada aplicativo.
			 * 
			 * Por exemplo. Se você estiver implementado um aplicativo de envio de notificações por
			 * SMS, a seguir você deverá implementar todo o código para envio de SMS.
			 * Se você estiver implementando um aplicativo de envio de notificações via PUSH,
			 * a seguir você deve implementar o código para envio de mensagens PUSH.
			 * 
			 * Você pode criar outras classes e métodos a vontade. Apenas certifique-se
			 * de colocar as chamadas ao seu código aqui.
			 *
			 * O código abaixo pode ser útil para estudar os atributos dos objetos
			 * que compõe a estrutura de uma notificação.
			 */
			
			/*FORMATA O TEXTO DA NOTIFICAÇÃO*/
			
			/* Usuários que enviaram a notificação. ex: Fulano, Beltrano e Ciclano */
			int sendersSize = alert.getSenders().size();
			String senders = "";
			
			if (sendersSize > 0) {
				senders = (sendersSize == 1) ?
						alert.getSenders().get(0).getUser().getFullName() :
						alert.getSenders().get(0).getUser().getFullName() + " e mais " + (alert.getSenders().size()-1);
			}
			
			/* Evento que gerou a notificação. ex: curtiu */
			String event = "";
			
			if (sendersSize > 1) {
				event = alert.getEvent().getGroupDescription();
			} else {
				event = alert.getEvent().getSingleDescription();
			}
			
			/* Objeto sobre o qual o evento foi executado */
			String object = "";
			
			if (alert.getObject() != null) {
				
				//Descrição do tipo do objeto: ex. 'o post'
				if (alert.getObject().getTypeDescription() != null) {
					object += alert.getObject().getTypeDescription() + " ";
				}
				
				//Descrição (nome) do objeto: ex. 'Olha que post bacana...'
				object += "\"" + alert.getObject().getDescription() + "\"";
				
				//Detalhes do objeto
				if (alert.getObject().getObjectDetail() != null) {
					object += " " +alert.getObject().getObjectDetail();
				}
				
			}
			
			/* Lugar onde o evento foi gerado */
			String place = "";
			
			if (alert.getPlace() != null) {
				
				//Descrição (nome) do lugar: 'Comunidade do Fluig'
				place += "in \"" + alert.getPlace().getDescription() + "\"";
				
				//Detalhes do lugar. ex: que você é participante
				if (alert.getPlace().getObjectDetail() != null) {
					place += " " + alert.getPlace().getObjectDetail();
				}
				
			}
			
			/* Concatena os objetos do alerta e gera o texto final */
			final String alertText = new StringBuilder().
					append(senders).
					append(" ").
					append(event).
					append(" ").
					append(object).
					append(" ").
					append(place).
					append("\nMetadados: ").
					append(metadata).
					toString();
			
			/* Carrega o arquivo config.properties */
			final Properties config = new Properties();
			config.load(TxtSenderServiceBean.class.getClassLoader().getResourceAsStream("config.properties"));
			
			/* Recupera o diretório onde será salvo o arquivo de texto */
			final File alertsFolder = new File(config.getProperty("folder.path"));
			
			/* Salva o arquivo com o texto do alerta */
			final File alertFile = new File(alertsFolder, "notificacao_" + System.currentTimeMillis() + ".txt");
			
			try (final FileOutputStream fop = new FileOutputStream(alertFile);) {
				fop.write(alertText.getBytes());
				fop.flush();
			}
		
		} catch (Exception ex) {
			LOG.error("Could not send alert to a txt file: " + ex.getMessage());
		}
		
	}

}
